@@ -7,16 +7,16 @@ import os
 import subprocess
 
 from dataclasses import dataclass
-from datetime import date, datetime, timedelta
+from datetime import date, datetime, time, timedelta
 from pathlib import Path
-from typing import Dict, Iterable, List, Set, Tuple
+from typing import Any, Callable, Dict, Iterable, List, Set, Tuple
 
 import yacf
 from pretty_tables import PrettyTables
 
 
 # app specific constants
-__version__ = (0, 1, 5)
+__version__ = (0, 1, 7)
 __app__ = "tl"
 
 # regular constants
@@ -25,7 +25,9 @@ CONFIGS = [
     '/etc/tl.toml',
     './config.toml',
 ]
-WRITE_ACCESS = ['reset', 'restart', 'start', 'stop']
+
+READ_ACCESS = ['stats']
+WRITE_ACCESS = ['add', 'reset', 'restart', 'start', 'stop']
 
 
 class IllegalOperation(Exception):
@@ -138,6 +140,24 @@ class RecordSet:
     def size(self) -> int:
         return len(self._recs)
 
+    def add_rec(self, _from: datetime, _to: datetime, tags: Set):
+        """Appends a new record to the record set. Swaps with the last element
+        if this is open.
+
+        :param _from: Start time of the record
+        :type _from: datetime
+        :param _to: End time of the record
+        :type _to: datetime
+        """
+        rec = Record(_from, _to, tags)
+
+        if not self.closed:
+            curr = self._recs.pop()
+            self._recs.append(rec)
+            self._recs.append(curr)
+        else:
+            self._recs.append(rec)
+
     def generate_stats(self) -> Dict:
         """Generates a dictionary and sums up the durations per tag.
 
@@ -230,7 +250,7 @@ WARNING: This tool is just a prototype of a rapid development process. The final
     parser.add_argument(
         "cmd",
         nargs=1,
-        choices=["start", "stats", "stop", "reset", "restart"],
+        choices=READ_ACCESS + WRITE_ACCESS,
         help="Start or close a time recording",
     )
 
@@ -239,7 +259,7 @@ WARNING: This tool is just a prototype of a rapid development process. The final
         '--date',
         nargs='?',
         type=date.fromisoformat,
-        help="Define a workdate (Format: YYYY-MM-DD)",
+        help="Define a workdate, in YYYY-MM-DD)",
         default=date.fromisoformat(date.today().isoformat()),
     )
 
@@ -247,7 +267,7 @@ WARNING: This tool is just a prototype of a rapid development process. The final
         "-t",
         "--tags",
         nargs="+",
-        default=[],
+        default="default",
     )
 
     parser.add_argument(
@@ -255,6 +275,22 @@ WARNING: This tool is just a prototype of a rapid development process. The final
         action=argparse.BooleanOptionalAction,
     )
 
+    # TODO: Consider using a subparser for the different commands, especially the <add> command
+    #       https://docs.python.org/3/library/argparse.html?highlight=argparse#argparse.ArgumentParser.add_subparsers
+    parser.add_argument(
+        "--from",
+        type=time.fromisoformat,
+        help="Defines the start time when using the <add> command, in HH[:MM[:SS]]",
+        dest="_from",
+    )
+
+    parser.add_argument(
+        "--to",
+        type=time.fromisoformat,
+        help="Defines the end time when using the <add> command, in HH[:MM[:SS]]",
+        dest="_to",
+    )
+
     return parser
 
 
@@ -325,6 +361,36 @@ def read_recs(path: str) -> RecordSet:
     return recs
 
 
+def read_input(msg: str, of_type: Callable = str) -> Any:
+    """Prints a question and requests some user input. If a 'read_as' type is
+    defined, it tries to parse the value into the type and repeatedly asks for
+    input, if the input is invalid.
+
+    :param msg: Message to be prompted to the user
+    :type msg: str
+    :param of_type: Function to be used to parse user input, defaults to str
+    :type of_type: Callable, optional
+    :return: Parsed input
+    :rtype: Any
+    """
+    msg = msg.strip(' ')
+
+    if not msg[-1] in [':', '?']:
+        msg += ':'
+
+    inp = None
+
+    while inp is None:
+        try:
+            inp = input(msg)
+            inp = of_type(inp)
+        except Exception:
+            print(f"Invalid input. Please enter a valid '{of_type.__name__}'")
+            inp = None
+
+    return inp
+
+
 def shell(*args, cwd: str = None) -> Tuple[str, str]:
     """Performs a shell command and returns the piped STDERR & STDOUT.
     Args has to be an Iterable[Iterable[str]].
@@ -396,8 +462,6 @@ def main():
         # cfg.get('git', {})['enabled'] = args.git
         cfg['git.enabled'] = args.git
 
-    # TODO: Make workdate configurable with a CLI option
-    # workdate = date.today()
     workdate = args.date
 
     path = os.path.join(
@@ -429,16 +493,28 @@ def main():
 
     try:
         cmd = args.cmd[0]
+        
+        # Parse tags:
+        tags = args.tags.split(',')
 
-        if cmd == "reset":
+        if cmd == "add":
+            # prepare datetime stamps for _from and _to
+            _from = datetime.combine(workdate, args._from)
+            _to = datetime.combine(workdate, args._to)
+
+            if _from is None or _to is None:
+                log.error("Missing arguments for the options --from and --to. Exiting...")
+            else:
+                recs.add_rec(_from, _to, tags)
+        elif cmd == "reset":
             recs.reset_rec()
         elif cmd == "restart":
             recs.restart_rec()
         elif cmd == "start":
-            if not args.tags:
-                tags = {'default'}
-            else:
-                tags = set(args.tags[0].split(','))
+            #if not args.tags:
+            #    tags = {'default'}
+            #else:
+            #    tags = set(args.tags[0].split(','))
             recs.start_rec(tags)
         elif cmd == "stats":
             # TODO: Add more options, e.g:
@@ -471,8 +547,6 @@ def main():
             log.error("Failed to write to file.")
             log.debug("Details: ", exc_info=e)
 
-    # TODO: Add git push
-
     if has_write and cfg.get('git.enabled'):
         log.debug("Trying to push latest changes to git.")
         out, err = shell(
